﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Buddy.BehaviorTree;
using Buddy.CommonBot;
using Buddy.Swtor;

using WingIt.Routines;

using Action = Buddy.BehaviorTree.Action;


namespace WingIt
{
    class CommonSubBT
    {
        /// <summary>
        /// Creates sub-BT for use in a PrioritySelector...
        /// Returns RunStatus.Success to the BT parent if we're in the process of breaking crouch/cover.
        /// Otherwise, returns RunStatus.Failure indicating no action was necessary.
        /// </summary>
        /// <returns>Composite for use in a PrioritySelector</returns>
        public static Composite     CreateBTPS_BreakCrouchOrCover()
        {
            return new Decorator(ctx => Global.IsInCover,
                new Sequence(
                    // The only technique the SWtOR client provides to break crouch/cover is to 'bump move'.
                    // Unfortunately, this may appear to the user as a stutter-step by the toon.
                    new Action(ctx => Buddy.Swtor.Movement.Move(MovementDirection.Backward, TimeSpan.FromMilliseconds(50))),
                    // Throttle unsuccessful attempts to break Cover/Crouch...
                    new Wait(TimeSpan.FromMilliseconds(750), abortWaitingWhen => !Global.IsInCover, new Action())
                    ));
        }

        /// <summary>
        /// Creates sub-BT for use in a PrioritySelector...
        /// Returns RunStatus.Success to the BT parent if we're in the process of stopping a cast.
        /// Otherwise, returns RunStatus.Failure indicating no action was necessary.
        /// </summary>
        /// <returns>Composite for use in a PrioritySelector</returns>
        public static Composite     CreateBTPS_StopCasting()
        {
            return new Decorator(ret => BuddyTor.Me.IsCasting,
                new Sequence(
                    new Action(ctx =>  { using(BuddyTor.Memory.AcquireFrame()) { BuddyTor.Me.CallScript<bool>(true, "AbilityCancel"); }}),
                    // Throttle unsuccessful attempts to stop casting...
                    new Wait(TimeSpan.FromMilliseconds(750), abortWaitingWhen => !BuddyTor.Me.IsCasting, new Action())
                    ));
        }

        /// <summary>
        /// Creates sub-BT for use in a PrioritySelector...
        /// Returns RunStatus.Success to the BT parent if we're in the process of ceasing toon movement.
        /// Otherwise, returns RunStatus.Failure indicating no action was necessary.
        /// </summary>
        /// <returns>Composite for use in a PrioritySelector</returns>
        public static Composite     CreateBTPS_StopMoving()
        {
            return new Decorator(ctx => BuddyTor.Me.IsMoving,
                new Sequence(
                    CommonBehaviors.MoveStop(),
                    // Throttle unsuccessful attempts to stop movement (e.g., sliding down a hill or somesuch)...
                    new Wait(TimeSpan.FromMilliseconds(750), abortWaitingWhen => !BuddyTor.Me.IsMoving, new Action())
                    ));
        }
    }
}
