﻿using System.Linq;
using Buddy.BehaviorTree;
using Buddy.CommonBot;
using Buddy.CommonBot.Logic;
using Buddy.Swtor;
using WingIt.Dynamics;

namespace WingIt.Routines
{
    public static class OperativeLethality
    {
        #region Constants
        private const double RescueResourceLevel = 45;
        private const double DefaultSelfHealingLevel = 75;
        private const double DefaultCompanionHealingLevel = 45;
        private const double DefaultCompanionIsDeadMeHealingLevel = 50;
        private const double CriticalMeHealingLevel = 20;
        private const double CriticalCompanionHealingLevel = 20;
        private const double CriticalResourceLevel = 25;
        #endregion

        #region Statics
        private static bool MeHasTacticalAdvantage { get { return BuddyTor.Me.HasBuff("Tactical Advantage"); } }
        #endregion

        [Behavior(BehaviorType.Pull)]
        [Class(CharacterClass.Agent, AdvancedClass.Operative, SkillTreeId.OperativeLethality)]
        public static Composite OperativeLethalityPull()
        {
            return new PrioritySelector(
                Movement.StopInRange(Global.RangeDistance),
                Movement.MoveToLineOfSight(),
                Spell.WaitForCast(),

                Spell.Cast("Rifle Shot")
            );
        }

        [Behavior(BehaviorType.Combat)]
        [Class(CharacterClass.Agent, AdvancedClass.Operative, SkillTreeId.OperativeLethality)]
        public static Composite OperativeLethalityCombat()
        {
            return new PrioritySelector(
                Movement.StopInRange(Global.RangeDistance),
                Spell.WaitForCast(),

                // CD's
                Spell.Cast("Shield Probe", ret => BuddyTor.Me, ret => BuddyTor.Me.HealthPercent <= DefaultSelfHealingLevel), // Absorbs DMG for 15s
                Spell.Cast("Evasion", ret => BuddyTor.Me, ret => BuddyTor.Me.HealthPercent <= CriticalMeHealingLevel), // Immune to DMG for 3s
                Spell.Cast("Stim Boost", ret => BuddyTor.Me, ret => MeHasTacticalAdvantage), // Restores Energy over time
                Spell.Cast("Adrenaline Probe", ret => BuddyTor.Me, ret => BuddyTor.Me.ResourceStat <= RescueResourceLevel), // Restores 50 Energy
                Spell.Cast("Distraction", ret => BuddyTor.Me.CurrentTarget.Range <= 1f && Global.RequiresCrowdControl), // Interrupt
                Spell.Cast("Flash Bang", ret => BuddyTor.Me.CurrentTarget.Range <= 1f && Global.RequiresCrowdControl), // Blind/Interrupt
                Spell.Cast("Debilitate", ret => BuddyTor.Me.CurrentTarget.Range <= Global.MeleeDistance && Global.RequiresCrowdControl), // Stun

                // Do we need to heal ourselves?
                new Decorator(ret => BuddyTor.Me.HealthPercent <= DefaultSelfHealingLevel,
                    new PrioritySelector(
                        ctx => BuddyTor.Me, // Set us as context please. Kthx!

                        Spell.Cast("Kolto Infusion", ret => BuddyTor.Me, ret => MeHasTacticalAdvantage),
                        Spell.Cast("Kolto Injection", ret => BuddyTor.Me, ret => AbilityManager.CanCast("Kolto Injection", BuddyTor.Me)),
                        Spell.Cast("Kolto Infusion", ret => BuddyTor.Me, ret => MeHasTacticalAdvantage),
                        Spell.Cast("Diagnostic Scan", ret => BuddyTor.Me, ret => BuddyTor.Me.ResourceStat < CriticalResourceLevel)
                        )),

                // Do we need to heal our companion?
                new Decorator(ret => BuddyTor.Me.Companion != null && !BuddyTor.Me.Companion.IsDead && BuddyTor.Me.Companion.HealthPercent <= DefaultCompanionHealingLevel,
                    new PrioritySelector(
                        Spell.Cast("Kolto Infusion", ret => BuddyTor.Me.Companion, ret => MeHasTacticalAdvantage),
                        Spell.Cast("Kolto Injection", ret => BuddyTor.Me.Companion, ret => AbilityManager.CanCast("Kolto Injection", BuddyTor.Me.Companion)),
                        Spell.Cast("Kolto Infusion", ret => BuddyTor.Me.Companion, ret => MeHasTacticalAdvantage),
                        Spell.Cast("Diagnostic Scan", ret => BuddyTor.Me.Companion, ret => BuddyTor.Me.ResourceStat < CriticalResourceLevel)
                        )),

                        //Spell.Cast("Hidden Strike", ret => BuddyTor.Me.IsStealthed), // requires behind Target
                        Spell.Cast("Eviscerate", ret => Global.RequiresCrowdControl),
                        Spell.Cast("Backstab"), // Requires behind Target
                        Spell.Cast("Shiv"),


                        //Spell.Cast("Orbital Strike", ret => Helpers.MobsAround(Global.MeleeDistance) >= 3),
                        Spell.Cast("Corrosive Grenade", ret => Helpers.MobsAround(Global.MeleeDistance) >= 3),
                        Spell.Cast("Fragmentation Grenade", ret => Helpers.MobsAround(Global.MeleeDistance) >= 3),
                        Spell.Cast("Carbine Burst", ret => Helpers.MobsAround(Global.MeleeDistance) >= 3 && BuddyTor.Me.CurrentTarget.Range <= 1f && MeHasTacticalAdvantage),

                        Spell.Cast("Corrosive Dart", ret => !BuddyTor.Me.CurrentTarget.HasDebuff("Poisoned (Tech)") && Global.RequiresCrowdControl),

                        Spell.Cast("Headshot", ret => BuddyTor.Me.CurrentTarget.IsStunned),

                        Spell.Cast("Crouch", ret => !Global.IsInCover),

                        Spell.Cast("Explosive Probe"),
                        Spell.Cast("Snipe"),
                        Spell.Cast("Overload Shot", ret => !Global.IsInCover),
                        Spell.Cast("Rifle Shot", ret => BuddyTor.Me.ResourceStat < CriticalResourceLevel),

                Movement.MoveTo(ret => BuddyTor.Me.CurrentTarget.Position, Global.RangeDistance)
                );
        }


        [Behavior(BehaviorType.OutOfCombat)]
        [Class(CharacterClass.Agent, AdvancedClass.Operative, SkillTreeId.OperativeLethality)]
        public static Composite OperativeLethalityPostCombat()
        {
            // PostCombat handling prevents the 'stuck handler dance' if combat ends in crouch/cover
            return (CommonSubBT.CreateBTPS_BreakCrouchOrCover());
        }
    }
}