﻿using System;
using System.Linq;
using Buddy.BehaviorTree;
using Buddy.Common;
using Buddy.CommonBot;
using Buddy.Swtor;
using WingIt.Dynamics;
using Action = Buddy.BehaviorTree.Action;

namespace WingIt.Routines
{
    //Neo93
    public static class SageTelekinetics
    {
        [Behavior(BehaviorType.Pull)]
        [Class(CharacterClass.Consular, AdvancedClass.Sage, SkillTreeId.SageTelekinetics)]
        public static Composite SageTelekineticPull()
        {
            return SageTelekineticCombat();
        }

        [Behavior(BehaviorType.Combat)]
        [Class(CharacterClass.Consular, AdvancedClass.Sage, SkillTreeId.SageTelekinetics)]
        public static Composite SageTelekineticCombat()
        {
            return new PrioritySelector(
                Spell.WaitForCast(),
                Movement.StopInRange(Global.RangeDistance),

                //General
                Spell.Cast("Force of Will", castWhen => BuddyTor.Me.IsStunned),
                Spell.Cast("Force Potency"),//Buff for 2 spells
                Spell.Cast("Mental Alacrity"),

                //CC
                Spell.Cast("Force Stun", ret => Global.RequiresCrowdControl),
                Spell.Cast("Force Wave", ret => Helpers.MobsAround(Global.MeleeAoEDistance) >= 3 && Helpers.Targets.Count() >= 3),

                //Healing
                Spell.Cast("Force Armor", on => Helpers.Tank, castWhen => !Global.IsInGroup && !Helpers.Tank.HasBuff("Force Armor") && !Helpers.Tank.HasDebuff("Force-imbalance")),//Shield
                Spell.Cast("Force Armor", on => Helpers.LowestHealthPlayer, castWhen => !Global.IsInGroup && !Helpers.LowestHealthPlayer.HasBuff("Force Armor") && !Helpers.LowestHealthPlayer.HasDebuff("Force-imbalance") && Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth),//Shield
                Spell.Cast("Rejuvenate", on => Helpers.LowestHealthPlayer, castWhen => !Global.IsInGroup && Helpers.LowestHealthPlayer.HealthPercent <= Global.HighHealth && !Helpers.LowestHealthPlayer.HasBuff("Rejuvenate")),//Low Heal & HoT
                Spell.Cast("Healing Trance", on => Helpers.LowestHealthPlayer, castWhen => !Global.IsInGroup && Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth),//Low Heal 

                //Offensive
                Spell.Cast("Weaken Mind", ret => !BuddyTor.Me.CurrentTarget.HasDebuff("Weaken Mind (Force)")),
                Spell.Cast("Disturbance", ret => (AbilityManager.HasAbility("Concentration") && !Helpers.HasBuffStacks(BuddyTor.Me, "Concentration", 3))),
                Spell.Cast("Mind Crush"),
                Spell.Cast("Turbulence"),
                Spell.Cast("Telekinetic Wave", on => BuddyTor.Me.CurrentTarget, ret => Helpers.MobsAround(0.8f) >= 3 || BuddyTor.Me.HasBuff("Tidal Force")),
                Spell.Cast("Telekinetic Throw", ret => (AbilityManager.HasAbility("Psychic Projection") && BuddyTor.Me.HasBuff("Psychich Projection"))|| !AbilityManager.HasAbility("Psychic Projection")),
                Spell.Cast("Project"),
                Spell.Cast("Disturbance"),
                
                //Movement
                Movement.MoveTo(ret => BuddyTor.Me.CurrentTarget.Position, Global.RangeDistance)
                );
        }

        [Behavior(BehaviorType.OutOfCombat)]
        [Class(CharacterClass.Consular, AdvancedClass.Sage, SkillTreeId.SageTelekinetics)]
        public static Composite SageTelekineticsOutOfCombat()
        {
            return new PrioritySelector(
                    Spell.Cast("Force Speed", ret => BuddyTor.Me.IsMoving),
                    new Decorator(ret => Helpers.LowestHealthPlayer != null && Helpers.LowestHealthPlayer.HealthPercent <= 95,
                        new PrioritySelector(
                            Spell.Cast("Rejuvenate", on => Helpers.LowestHealthPlayer, j => Helpers.LowestHealthPlayer.HealthPercent <= Global.HighHealth && !Helpers.LowestHealthPlayer.HasBuff("Rejuvenate")),//Low Heal & HoT
                            Spell.Cast("Healing Trance", on => Helpers.LowestHealthPlayer, k => Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth)//Low Heal 
                )));
        }
    }
}