﻿using System.Linq;
using Buddy.BehaviorTree;
using Buddy.Common;
using Buddy.CommonBot;
using Buddy.Swtor;
using Buddy.Swtor.Objects;
using WingIt.Dynamics;

namespace WingIt.Routines
{
    //09.05.2012 Pios
    public static class SorcererMadness
    {
        [Behavior(BehaviorType.Pull)]
        [Class(CharacterClass.Inquisitor, AdvancedClass.Sorcerer, SkillTreeId.SorcererMadness)]
        public static Composite SorcererMadnessPull()
        {
            return new PrioritySelector(
                Movement.MoveTo(ret => BuddyTor.Me.CurrentTarget.Position, 29f / 10f),
                Spell.CastOnGround("Death Field",
                                   castWhen => BuddyTor.Me.ResourceStat >= 100,
                                   location => BuddyTor.Me.CurrentTarget.Position),
                Spell.Cast("Affliction"),
                SorcererMadnessCombat()
                );
        }

        [Behavior(BehaviorType.Combat)]
        [Class(CharacterClass.Inquisitor, AdvancedClass.Sorcerer, SkillTreeId.SorcererMadness)]
        public static Composite SorcererMadnessCombat()
        {
            return new PrioritySelector(
                Spell.WaitForCast(),
                Movement.StopInRange(Global.RangeDistance),

                //General
                Spell.Cast("Unbreakable Will", castWhen => BuddyTor.Me.IsStunned),
                Spell.Cast("Recklessness"),//Buff for 2 spells

                //CC
                Spell.Cast("Electrocute", ret => Global.RequiresCrowdControl),
                Spell.Cast("Overload", ret => Helpers.MobsAround(Global.MeleeAoEDistance) >= 3 && Helpers.Targets.Count() >= 3),

                //Healing
                Spell.Cast("Static Barrier", on => Helpers.Tank, ret => !Global.IsInGroup && !Helpers.Tank.HasBuff("Static Barrier") && !Helpers.Tank.HasDebuff("Deionized")),//Shield
                Spell.Cast("Static Barrier", on => Helpers.LowestHealthPlayer, ret => !Global.IsInGroup && !Helpers.LowestHealthPlayer.HasBuff("Static Barrier") && !Helpers.LowestHealthPlayer.HasDebuff("Deionized") && Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth),//Shield
                Spell.Cast("Resurgence", on => Helpers.LowestHealthPlayer, ret => !Global.IsInGroup && Helpers.LowestHealthPlayer.HealthPercent <= Global.HighHealth && !Helpers.LowestHealthPlayer.HasBuff("Resurgence")),//Low Heal & HoT
                Spell.Cast("Innervate", on => Helpers.LowestHealthPlayer, ret => !Global.IsInGroup && Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth),//Low Heal & HoT

                //Offensive
                Spell.CastOnGround("Death Field", ret => BuddyTor.Me.CurrentTarget.Distance <= 0.8f, location => BuddyTor.Me.CurrentTarget.Position),
                Spell.Cast("Affliction", ret => !BuddyTor.Me.CurrentTarget.HasDebuff("Affliction (Force)")),
                Spell.Cast("Creeping Terror", ret => !BuddyTor.Me.CurrentTarget.HasDebuff("Creeping Terror")),
                Spell.Cast("Crushing Darkness", ret => BuddyTor.Me.HasBuff("Wrath")),
                Spell.Cast("Force Lightning"),
                Spell.Cast("Shock"),
                Spell.Cast("Lightning Strike", ret => (AbilityManager.HasAbility("Wrath") && BuddyTor.Me.HasBuff("Wrath")) || !AbilityManager.HasAbility("Wrath")),

                //Movement
                Movement.MoveTo(ret => BuddyTor.Me.CurrentTarget.Position, Global.RangeDistance)
                );
        }

        [Behavior(BehaviorType.OutOfCombat)]
        [Class(CharacterClass.Inquisitor, AdvancedClass.Sorcerer, SkillTreeId.SorcererMadness)]
        public static Composite SorcererMadnessOutOfCombat()
        {
            return new PrioritySelector(
                    Spell.Cast("Force Speed", ret => BuddyTor.Me.IsMoving),
                    new Decorator(ret => Helpers.LowestHealthPlayer != null && Helpers.LowestHealthPlayer.HealthPercent <= 95,
                        new PrioritySelector(
                            Spell.Cast("Resurgence", on => Helpers.LowestHealthPlayer, j => Helpers.LowestHealthPlayer.HealthPercent <= Global.HighHealth && !Helpers.LowestHealthPlayer.HasBuff("Resurgence")),//Low Heal & HoT
                            Spell.Cast("Innervate", on => Helpers.LowestHealthPlayer, k => Helpers.LowestHealthPlayer.HealthPercent <= Global.MidHealth)//Low Heal 
                )));
        }

    }
}
