﻿using System;
using Buddy.BehaviorTree;
using Buddy.CommonBot;
using Buddy.Swtor;
using System.Linq;
using WingIt.Dynamics;
using Action = Buddy.BehaviorTree.Action;

namespace WingIt.Routines.Basic
{
    public class Smuggler
    {
        [Class(CharacterClass.Smuggler)]
        [Behavior(BehaviorType.Combat)]
        public static Composite SmugglerCombat()
        {
            //Neo93 29.04.2012
            return new PrioritySelector(
                Spell.WaitForCast(),
                Movement.StopInRange(Global.RangeDistance),

                //Hawker 29/4
                Spell.Cast("Dirty Kick", castWhen => BuddyTor.Me.ResourceStat >= 10 && BuddyTor.Me.CurrentTarget.Distance <= Global.MeleeDistance),
                Spell.Cast("Blaster Whip", castWhen => BuddyTor.Me.ResourceStat >= 10 && BuddyTor.Me.CurrentTarget.Distance <= Global.MeleeDistance),

                //General
                Spell.Cast("Crouch", onUnit => BuddyTor.Me, castWhen => BuddyTor.Me.CurrentTarget.InLineOfSight && !Global.IsInCover && BuddyTor.Me.CurrentTarget.Distance <= Global.RangeDistance && BuddyTor.Me.InCombat),
                Spell.Cast("Escape ", castWhen => BuddyTor.Me.IsStunned),//Insignia

                // AOE
                Spell.Cast("Thermal Grenade", castWhen => Helpers.Targets.Count() >= 3 && BuddyTor.Me.ResourceStat >= 80),

                //CC
                Spell.Cast("Flash Grenade", onUnit =>
                    {
                        var previousTarget = BuddyTor.Me.CurrentTarget;
                        return
                            Helpers.Targets.FirstOrDefault(
                                t =>
                                t != previousTarget && (t.Toughness == CombatToughness.Strong)) ??
                            Helpers.Targets.FirstOrDefault(t => t != previousTarget);
                    }, castWhen => Helpers.Targets.Count() >= 3),

                //Offensive
                //Spell.Cast("Flurry of Bolts", castWhen => BuddyTor.Me.CurrentTarget.HasDebuff("Sabotage Charge")),//No Resource needed

                Spell.Cast("Sabotage Charge", castWhen => Global.IsInCover && BuddyTor.Me.ResourceStat >= 80 && BuddyTor.Me.CurrentTarget.Distance <= Global.RangeDistance),//Grenade
                Spell.Cast("Vital Shot", castWhen => BuddyTor.Me.ResourceStat >= 80 && BuddyTor.Me.CurrentTarget.Toughness != CombatToughness.Standard && BuddyTor.Me.CurrentTarget.Toughness != CombatToughness.Weak && !BuddyTor.Me.CurrentTarget.HasBuff("Bleeding")),//Bleed
                Spell.Cast("Dirty Kick", castWhen => BuddyTor.Me.CurrentTarget.Distance <= 0.4f && BuddyTor.Me.ResourceStat >= 70),//Stun
                Spell.Cast("Blaster Whip", castWhen => BuddyTor.Me.CurrentTarget.Distance <= 0.4f && BuddyTor.Me.ResourceStat >= 75),//melee attack
                Spell.Cast("Quick Shot", castWhen => BuddyTor.Me.ResourceStat >= 77 && BuddyTor.Me.CurrentTarget.Distance <= 1f),
                Spell.Cast("Charged Burst", castWhen => Global.IsInCover && BuddyTor.Me.ResourceStat >= 80 && BuddyTor.Me.CurrentTarget.Distance <= Global.RangeDistance),
                Spell.Cast("Flurry of Bolts"),//No Resource needed

                Movement.MoveTo(whereTo => BuddyTor.Me.CurrentTarget.Position, Global.RangeDistance)
                );
        }

        [Class(CharacterClass.Smuggler)]
        [Behavior(BehaviorType.Pull)]
        public static Composite SmugglerPull()
        {
            return SmugglerCombat();
        }

        [Class(CharacterClass.Smuggler)]
        [Behavior(BehaviorType.OutOfCombat)]
        public static Composite SmugglerPostCombat()
        {
            // PostCombat handling prevents the 'stuck handler dance' if combat ends in crouch/cover
            return (CommonSubBT.CreateBTPS_BreakCrouchOrCover());
        }
    }
}