﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Buddy.Swtor;

namespace WingIt
{
    class Tunables
    {
        public delegate bool        PredicateDelegate();
        public delegate float       NormalizedResourceDelegate();

        public class ClassTunables
        {
            public string               RejuvenateName              = "UNDEFINED";
            public string               SelfBuffName                = "UNDEFINED";
            public PredicateDelegate    IsRejuvenationNeeded        = () => { return (false); };
            public PredicateDelegate    IsRejuvenationComplete      = () => { return (true); };
            public NormalizedResourceDelegate   NormalizedResource  = () => { return (0.0f); };
        };


        private static readonly ClassTunables   CompanionTunables = new ClassTunables()
        {
            IsRejuvenationNeeded = () => { return (Helpers.IsCompanionInUse() && !Helpers.Companion.IsDead && (Helpers.Companion.HealthPercent < 70)); },
            IsRejuvenationComplete = () => { return (!Helpers.IsCompanionInUse() || Helpers.Companion.IsDead || (Helpers.Companion.HealthPercent >= 95)); }
        };

        // This unifies handling of class generic buffing & healing...
        // Previously, various (disparate) constants were used inconsistently thoughout the code.
        private static readonly Dictionary<CharacterClass, ClassTunables>  ToonTunables = new Dictionary<CharacterClass, ClassTunables>()
        {
            // Republic
            { CharacterClass.Knight, 
                new ClassTunables() {
                    RejuvenateName = "Introspection",
                    SelfBuffName = "Force Might",
                    IsRejuvenationNeeded = () => { return (BuddyTor.Me.HealthPercent < 70); },
                    IsRejuvenationComplete = () => { return (BuddyTor.Me.HealthPercent >= 95); },
                    NormalizedResource = () => { return (BuddyTor.Me.ResourceStat); }
                }},

            { CharacterClass.Consular,
                new ClassTunables() {
                    RejuvenateName = "Meditation",
                    SelfBuffName = "Force Valor",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat < 70)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat >= 95)); },
                    NormalizedResource = () => { return (BuddyTor.Me.ResourceStat); }
                }},

            { CharacterClass.Smuggler,
                new ClassTunables() {
                    RejuvenateName = "Recuperate",
                    SelfBuffName = "Lucky Shots",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat < 70)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat >= 95)); },
                    NormalizedResource = () => { return (BuddyTor.Me.ResourceStat); }
                }},

            { CharacterClass.Trooper,
                new ClassTunables() {
                    RejuvenateName = "Recharge and Reload",
                    SelfBuffName = "Fortification",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat < 5)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat > 9)); },
                    NormalizedResource = () => { return (BuddyTor.Me.ResourceStat * 100 / 12); }
                }},

            // Empire
            { CharacterClass.Warrior,
                new ClassTunables() {
                    RejuvenateName = "Channel Hatred",
                    SelfBuffName = "Unnatural Might",
                    IsRejuvenationNeeded = () => { return (BuddyTor.Me.HealthPercent < 70); },
                    IsRejuvenationComplete = () => { return (BuddyTor.Me.HealthPercent >= 95); },
                    NormalizedResource = () => { return (BuddyTor.Me.ResourceStat); }
                }},

            { CharacterClass.Inquisitor,
                new ClassTunables() {
                    RejuvenateName = "Seethe",
                    SelfBuffName = "Mark of Power",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat < 70)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat >= 95)); },
                    NormalizedResource = () => { return (BuddyTor.Me.ResourceStat); }
                }},

            { CharacterClass.Agent,
                new ClassTunables() {
                    RejuvenateName = "Recuperate",
                    SelfBuffName = "Coordination",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat < 70)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat >= 95)); },
                    NormalizedResource = () => { return (BuddyTor.Me.ResourceStat); }
                }},

            { CharacterClass.BountyHunter,
                new ClassTunables() {
                    RejuvenateName="Recharge and Reload",
                    SelfBuffName="Hunter's Boon",
                    IsRejuvenationNeeded = () => { return ((BuddyTor.Me.HealthPercent < 70) || (BuddyTor.Me.ResourceStat > 30)); },
                    IsRejuvenationComplete = () => { return ((BuddyTor.Me.HealthPercent >= 95) && (BuddyTor.Me.ResourceStat <= 5)); },
                    NormalizedResource = () => { return (100.0f - Math.Min(BuddyTor.Me.ResourceStat, 100.0f)); }
                }},

            // Boundary condition --
            { CharacterClass.Unknown,
                new ClassTunables() {
                    RejuvenateName="UNDEFINED",
                    SelfBuffName="UNDEFINED",
                    IsRejuvenationNeeded = () => { return (false); },
                    IsRejuvenationComplete = () => { return (true); },
                    NormalizedResource = () => { return (0.0f); }
                }}
        };


        public static ClassTunables     ForCompanion
        {
            get { return (CompanionTunables); }
        }

        public static ClassTunables     ForToon
        {
            get { return (ToonTunables[BuddyTor.Me.Class]); }
        }

        /// <summary>
        /// <para>Normalizes the <c>BuddyTor.Me.ResourceStat</c> to the closed interval of [0.0..100.0], where</para>
        /// <para>0.0 represents 'empty' or no resources available, and
        /// 100.0 represents full resources available.</para>
        /// <para>Notes:<list type="bullet">
        /// <item><description><para> * The Bounty Hunter's resource (heat) is reversed by this paradigm.  When heat is 100, there are no resources left, so 0.0 is returned by this method.</para></description></item>
        /// <item><description><para> * The Trooper's resource (ammo) is scaled up from 12 to represent a percentage of the ammo full ammo load.</para></description></item>
        /// </list></para>
        /// </summary>
        public static float             NormalizedResource
        {
            get { return (ForToon.NormalizedResource()); }
        }
    }
}
